/* ---------------------------------------------------------------------------- */
/*                  Atmel Microcontroller Software Support                      */
/*                       SAM Software Package License                           */
/* ---------------------------------------------------------------------------- */
/* Copyright (c) 2015, Atmel Corporation                                        */
/*                                                                              */
/* All rights reserved.                                                         */
/*                                                                              */
/* Redistribution and use in source and binary forms, with or without           */
/* modification, are permitted provided that the following condition is met:    */
/*                                                                              */
/* - Redistributions of source code must retain the above copyright notice,     */
/* this list of conditions and the disclaimer below.                            */
/*                                                                              */
/* Atmel's name may not be used to endorse or promote products derived from     */
/* this software without specific prior written permission.                     */
/*                                                                              */
/* DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR   */
/* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE   */
/* DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,      */
/* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,  */
/* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING         */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                           */
/* ---------------------------------------------------------------------------- */

/**
 *  \page gmac_uip_helloworld GMAC uIP Hello World Example
 *
 *  \section Purpose
 *
 *  This project implements a telnet hello-world example of the uIP stack.
 *  It makes the device to respond to telnet connections on port 1000.
 *
 *  \section Requirements
 *
 * This package can be used with SAMV71 Xplained Ultra board or SAME70 Xplained board.
 *
 *  \section Description
 *
 * Please refer to the uIP documentation for more information
 * about the TCP/IP stack, the hello-world example.
 *
 * By default, the example does not use DHCP.
 * If you want to use DHCP, please:
 * - Open file uip-conf.h and don't comment the line "#define UIP_DHCP_on".
 * - Include uip/apps/dhcps to compile.
 *
 *  \section Usage
 *
 *  -# Build the program and download it inside the board.
 *     Please refer to the Getting Started with SAM V71/E70 Microcontrollers.pdf
 *  -# On the computer, open and configure a terminal application
 *     (e.g. HyperTerminal on Microsoft Windows) with these settings:
 *    - 115200 baud rate
 *    - 8 bits of data
 *    - No parity
 *    - 1 stop bit
 *    - No flow control
 *  -# Connect an Ethernet cable between the evaluation board and the network.
 *      The board may be connected directly to a computer; in this case,
 *      make sure to use a cross/twisted wired cable such as the one provided
 *      with the evaluation kit.
 *  -# Start the application. It will display the following message on the terminal:
 *    \code
 *    -- GMAC uIP Hello World Example xxx --
 *    -- xxxxxx-xx
 *    -- Compiled: xxx xx xxxx xx:xx:xx --
 *    - MAC 3a:1f:34:08:54:05
 *    - Host IP 10.217.12.223
 *    - Router IP 10.217.12.1
 *    - Net Mask 255.255.255.0
 *    \endcode
 * -# Connect to the device IP address using telnet on port 1000:
 *    \code
 *    telnet 10.217.12.223 1000
 *    \endcode
 *    A greeting message will appear:
 *    \code
 *    Hello. What is your name?
 *    \endcode
 *
 * \note
 * Make sure the IP address of the device(EK board) and the computer are in the
 * same network.
 *
 *  \section References
 *  - gmac_uip_helloworld/main_g2.c
 *  - gmacb.h
 *  - gmacd.h
 *  - gmac.h
 *  - uip.h
 */

/** \file
 *
 *  This file contains all the specific code for the gmac_uip_helloworld example.
 *
 */

/*----------------------------------------------------------------------------
 *        Headers
 *----------------------------------------------------------------------------*/

#include "g1/libboard/board.h"

#include "g2/libuip/uip.h"
#include "g2/libuip/uip_arp.h"
#include "g2/libuip/gmac_tapdev.h"
#include "g2/libuip/timer.h"
#include "virt_eth.h"

/*---------------------------------------------------------------------------
 *         Variables
 *---------------------------------------------------------------------------*/
/** TWI clock frequency in Hz. */
#define TWCK            400000
/** Slave address of twi_eeprom AT24MAC.*/
#define AT24MAC_SERIAL_NUM_ADD  0x5F
/** Page size of an AT24MAC402 chip (in bytes)*/
#define PAGE_SIZE       16
/** Page numbers of an AT24MAC402 chip */
#define EEPROM_PAGES    16
/** EEPROM Pins definition */
#define BOARD_PINS_TWI_EEPROM PINS_TWI0
/** TWI0 peripheral ID for EEPROM device*/
#define BOARD_ID_TWI_EEPROM   ID_TWIHS0
/** TWI0 base address for EEPROM device */
#define BOARD_BASE_TWI_EEPROM TWIHS0

/* uIP buffer : The ETH header */
#define BUF ((struct uip_eth_hdr *)&uip_buf_g2[0])

/** TWI driver instance.*/
static Twid twid;

/** The PINs for TWI*/
static const Pin twiPins[]      = BOARD_PINS_TWI_EEPROM;
/* The MAC address used for demo */
static struct uip_eth_addr GMacAddress_g2 = {{0x3a, 0x1f, 0x34, 0x08, 0x54, 0x55}};

/* The IP address used for demo (ping ...) */
static uint8_t HostIpAddress_g2[4] = {192, 168, 1, 4 };

/* Set the default router's IP address. */
static const uint8_t RoutIpAddress_g2[4] = {192, 168, 1, 2 };
/* The NetMask address */
static const uint8_t NetMask_g2[4] = {255, 255, 255, 0};

#define VIRT_ETH_NUM_BUFFERS 3
char ve_rx_buf_g2[VIRT_ETH_NUM_BUFFERS][1500];
struct rxBufDesc ve_rx_buf_desc_g2[VIRT_ETH_NUM_BUFFERS];

/*----------------------------------------------------------------------------
 *        Local functions
 *----------------------------------------------------------------------------*/

/**
 * Initialize demo application
 */
static void _app_init_g2(void)
{
	printf("P: hello-world application init G2\n\r");
	hello_world_init_g2();

#ifdef __DHCPC_H__
	printf("P: DHCPC Init\n\r");
	dhcpc_init_g2(GMacAddress_g2.addr, 6);
#endif
}

/*----------------------------------------------------------------------------
 *        Exported functions
 *----------------------------------------------------------------------------*/

/**
 * uip_log_g2: Global function for uIP to use.
 * \param m Pointer to string that logged
 */
void uip_log_g2(char *m)
{
	TRACE_INFO("-uIP log- %s\n\r", m);
}

#ifdef __DHCPC_H__
/**
 * dhcpc_configured: Global function for uIP DHCPC to use,
 * notification of DHCP configuration.
 * \param s Pointer to DHCP state instance
 */
void dhcpc_configured_g2(const struct dhcpc_state *s)
{
	u8_t *pAddr;

	// Disable Interrupts
	asm("mov r0,0xff\n"
	"msr basepri,r0\n"
	"udf #0":::"r0");

	printf("\n\r");
	printf("=== DHCP Configurations G2 ===\n\r");
	pAddr = (u8_t *)s->ipaddr;
	printf("- IP   : %d.%d.%d.%d\n\r",
			pAddr[0], pAddr[1], pAddr[2], pAddr[3]);
	pAddr = (u8_t *)s->netmask;
	printf("- Mask : %d.%d.%d.%d\n\r",
			pAddr[0], pAddr[1], pAddr[2], pAddr[3]);
	pAddr = (u8_t *)s->default_router;
	printf("- GW   : %d.%d.%d.%d\n\r",
			pAddr[0], pAddr[1], pAddr[2], pAddr[3]);
	pAddr = (u8_t *)s->dnsaddr;
	printf("- DNS  : %d.%d.%d.%d\n\r",
			pAddr[0], pAddr[1], pAddr[2], pAddr[3]);
	printf("===========================\n\r\n");

	// Disable Interrupts
	asm("mov r0,0\n"
	"msr basepri,r0\n"
	"udf #0":::"r0");

	uip_sethostaddr(s->ipaddr);
	uip_setnetmask(s->netmask);
	uip_setdraddr(s->default_router);

#ifdef __RESOLV_H__
	resolv_conf(s->dnsaddr);
#else
	printf("DNS NOT enabled in the demo\n\r");
#endif
}
#endif

void hermesVirtEth_Handler_g2()
{
	return;
}

SysTickConfigure_g2(){
	(*(uint32_t*)0xE000E010) = 7;
	(*(uint32_t*)0xE000E014) = 1;
}

void SysTick_Handler_g2(void){
	extern volatile uint32_t dwClockTick_g2;
	dwClockTick_g2++;
}

/**
 *  \brief gmac_uip_helloworld example entry point.
 *
 *  \return Unused (ANSI-C compatibility).
 */
int main_g2(void)
{
	uip_ipaddr_t ipaddr;
	struct timer periodic_timer, arp_timer;
	uint32_t i;
	struct uip_eth_addr OrigiGMacAddr;

	/* Disable watchdog */
	WDT_Disable(WDT);

	// NAK Note: EnableDCache function was creating problems for hermes because
	// reads from the CCSIDR weren't working correctly. This doesn't affect the
	// app since the caches are already enabled on boot, so not worth trying to
	// debug right now.
	//SCB_EnableICache(); // COMMENTING FOR HERMES
	//SCB_EnableDCache(); // COMMENTING FOR HERMES

	printf("-- GMAC uIP Hello World Example %s --\n\r", SOFTPACK_VERSION);
	printf("-- %s\n\r", BOARD_NAME);
	printf("-- Compiled: %s %s With %s--\n\r", __DATE__, __TIME__ ,
			COMPILER_NAME);

	SysTickConfigure_g2();
#if 0
	/* Configure systick for 1 ms. */
	TimeTick_Configure_g2();

	/* Configure TWI pins. */
	PIO_Configure_g2(twiPins, PIO_LISTSIZE(twiPins));
	/* Enable TWI */
	PMC_EnablePeripheral_g2(BOARD_ID_TWI_EEPROM);
	TWI_ConfigureMaster_g2(BOARD_BASE_TWI_EEPROM, TWCK, BOARD_MCK);
	TWID_Initialize_g2(&twid, BOARD_BASE_TWI_EEPROM);

	/* Display MAC & IP settings */
	TWID_Read_g2(&twid, AT24MAC_SERIAL_NUM_ADD, 0x9A, 1, OrigiGMacAddr.addr, PAGE_SIZE,
				0);
#endif
	// NAK Commenting so we can set the MAC address manually
#if 0
	if ((OrigiGMacAddr.addr[0] == 0xFC) && (OrigiGMacAddr.addr[1] == 0xC2)
		&& (OrigiGMacAddr.addr[2] == 0x3D)) {
		for (i = 0; i < 6; i++)
			GMacAddress.addr[i] = OrigiGMacAddr.addr[i];
	}
#endif
	printf("-- MAC %x:%x:%x:%x:%x:%x\n\r",
			GMacAddress_g2.addr[0], GMacAddress_g2.addr[1], GMacAddress_g2.addr[2],
			GMacAddress_g2.addr[3], GMacAddress_g2.addr[4], GMacAddress_g2.addr[5]);


	// Hermes virtual ethernet driver setup
	for(i = 0; i < VIRT_ETH_NUM_BUFFERS; i++){
		ve_rx_buf_desc_g2[i].addr.bm.addrDW = &ve_rx_buf_g2[i][0];
	}
	ve_rx_buf_desc_g2[VIRT_ETH_NUM_BUFFERS-1].addr.bm.bWrap = 1;
	createVirtualNetworkInterface(GMacAddress_g2.addr, ve_rx_buf_desc_g2);


#ifndef __DHCPC_H__
	printf(" - Host IP  %d.%d.%d.%d\n\r",
			HostIpAddress_g2[0], HostIpAddress_g2[1], HostIpAddress_g2[2], HostIpAddress_g2[3]);
	printf(" - Router IP  %d.%d.%d.%d\n\r",
			RoutIpAddress_g2[0], RoutIpAddress_g2[1], RoutIpAddress_g2[2], RoutIpAddress_g2[3]);
	printf(" - Net Mask  %d.%d.%d.%d\n\r",
			NetMask_g2[0], NetMask_g2[1], NetMask_g2[2], NetMask_g2[3]);
#endif

	/* System devices initialize */
	//gmac_tapdev_setmac_g2((uint8_t *)GMacAddress.addr);
	//gmac_tapdev_init_g2();
//	clock_init_g2();
//	clock_init_g1();
	timer_set_g2(&periodic_timer, CLOCK_SECOND / 2);
	timer_set_g2(&arp_timer, CLOCK_SECOND * 10);

	/* Init uIP */
	uip_init_g2();

#ifdef __DHCPC_H__
	printf("P: DHCP Supported\n\r");
	uip_ipaddr(ipaddr, 0, 0, 0, 0);
	uip_sethostaddr(ipaddr);
	uip_ipaddr(ipaddr, 0, 0, 0, 0);
	uip_setdraddr(ipaddr);
	uip_ipaddr(ipaddr, 0, 0, 0, 0);
	uip_setnetmask(ipaddr);
#else
	/* Set the IP address of this host */
	uip_ipaddr(ipaddr, HostIpAddress_g2[0], HostIpAddress_g2[1],
				HostIpAddress_g2[2], HostIpAddress_g2[3]);
	uip_sethostaddr(ipaddr);

	uip_ipaddr(ipaddr, RoutIpAddress_g2[0], RoutIpAddress_g2[1],
				RoutIpAddress_g2[2], RoutIpAddress_g2[3]);
	uip_setdraddr(ipaddr);

	uip_ipaddr(ipaddr, NetMask_g2[0], NetMask_g2[1], NetMask_g2[2], NetMask_g2[3]);
	uip_setnetmask(ipaddr);
#endif
	uip_setethaddr(GMacAddress_g2);

	_app_init_g2();

	while (1) {
		//uip_len = gmac_tapdev_read_g2();
		uip_len_g2 = virtEthRead(uip_buf_g2, UIP_BUFSIZE);

		if (uip_len_g2 > 0) {
			if (BUF->type == htons_g2(UIP_ETHTYPE_IP)) {
				//printf("G2 Rx'd IP Pkt\r\n\n");
				uip_arp_ipin();
				uip_input();

				/* If the above function invocation resulted in data that
					should be sent out on the network, the global variable
					uip_len is set to a value > 0. */
				if (uip_len_g2 > 0) {
					uip_arp_out_g2();
					//gmac_tapdev_send_g2();
					gmac_tapdev_send_hermes( uip_buf_g2, uip_len_g2 );
				}
			} else if (BUF->type == htons_g2(UIP_ETHTYPE_ARP)) {
				
				printf("G2 Rx'd ARP Pkt\r\n\n");

				uip_arp_arpin_g2();

				/* If the above function invocation resulted in data that
					should be sent out on the network, the global variable
					uip_len is set to a value > 0. */
				if (uip_len_g2 > 0){
					printf("G2 responding to ARP\r\n\n");
					gmac_tapdev_send_hermes( uip_buf_g2, uip_len_g2 );
					//gmac_tapdev_send_g2();
				}
			}
		} else if (timer_expired_g2(&periodic_timer)) {
			timer_reset_g2(&periodic_timer);

			for (i = 0; i < UIP_CONNS; i++) {
				uip_periodic(i);

				/* If the above function invocation resulted in data that
					should be sent out on the network, the global variable
					uip_len is set to a value > 0. */
				if (uip_len_g2 > 0) {

					uip_arp_out_g2();
					//gmac_tapdev_send_g2();
					gmac_tapdev_send_hermes( uip_buf_g2, uip_len_g2 );
				}
			}

#if UIP_UDP

			for (i = 0; i < UIP_UDP_CONNS; i++) {
				uip_udp_periodic(i);

				/* If the above function invocation resulted in data that
					should be sent out on the network, the global variable
					uip_len is set to a value > 0. */
				if (uip_len_g2 > 0) {
					uip_arp_out_g2();
					gmac_tapdev_send_hermes(uip_buf_g2, uip_len_g2);
				}
			}

#endif /* UIP_UDP */

			/* Call the ARP timer function every 10 seconds. */
			if (timer_expired_g2(&arp_timer)) {
				timer_reset_g2(&arp_timer);
				uip_arp_timer_g2();
			}
		}
	}
}

